import GenGIS
import rpy2
import rpy2.robjects as robjects

### RunHeatmap generates a heatmap representation of the relative frequency of different sequence file attributes across all locations.
### Two parameters must be passed:
###    columnnToUse: a column heading from the sequence file
###    labelsToUse: a list object containing at least two different labels from the selected column.
###    countField: field containing the number of times a sequence type was identified at a given location
###
### Example usage: Heatmap.RunHeatmap("Phylum",["Bacteroides","Firmicutes"],"Count")

def RunHeatmap(columnToUse,labelsToUse,countField):
	r = robjects.r

	### Identify all active location layers
	locs = GenGIS.layerTree.GetLocationSetLayer(0).GetAllActiveLocationLayers()
	
	### Dictionary of sequence counts by location for each of the requested labels
	seqCounts = {}
	
	### Dictionary of total sequence counts by location for normalization
	allCounts = {}
	
	### Iterate through the sequence data and build the dictionary of frequencies
	for loc in locs:
		seqCounts[loc] = {}
		allCounts[loc] = 0
		for addLabel in labelsToUse:
			seqCounts[loc][addLabel] = 0
	
		### Some sequences may be disabled, ignore them
		seqLayers = loc.GetAllActiveSequenceLayers()

		for seqLayer in seqLayers:
			data = seqLayer.GetController().GetData()
			thisLabel = data[columnToUse]
			seqCount = float(data[countField])
			if thisLabel in labelsToUse:
				seqCounts[loc][thisLabel] += seqCount
			allCounts[loc] += seqCount
	
	numLocs = len(locs)
	numLabels = len(labelsToUse)
		
	### The column and row labels
	yVec = r["c"](robjects.StrVector(labelsToUse))
	xVec = r["c"](robjects.StrVector([locX.GetController().GetData()["Organ"] for locX in locs]))

	### Now build the R matrix
	preList = []
	for i in xrange(0,numLocs):
		for j in xrange(0,numLabels):
			preList.append(seqCounts[locs[i]][labelsToUse[j]] / float(allCounts[locs[i]]))

	comVec = r["c"](robjects.FloatVector(preList))
	m = r.matrix(comVec, nrow = numLocs, byrow = True)
	robjects.globalEnv["m"] = m
	robjects.globalEnv["xVec"] = xVec
	robjects.globalEnv["yVec"] = yVec
	
	### And finally, the heatmap
	r("heatmap(m, col = topo.colors(100), scale='none', labCol=yVec, labRow=xVec, margins=c(10,10),cexCol=1.4)")
	